@@ -1,21 +1,21 @@
 use crate::core::compiler::{CompileKind, RustcTargetData};
 use crate::core::dependency::DepKind;
-use crate::core::resolver::{HasDevUnits, Resolve, ResolveOpts};
+use crate::core::package::SerializedPackage;
+use crate::core::resolver::{features::CliFeatures, HasDevUnits, Resolve};
 use crate::core::{Dependency, Package, PackageId, Workspace};
 use crate::ops::{self, Packages};
 use crate::util::interning::InternedString;
 use crate::util::CargoResult;
 use cargo_platform::Platform;
 use serde::Serialize;
-use std::collections::HashMap;
+use std::collections::BTreeMap;
 use std::path::PathBuf;
+use toml_edit::easy as toml;
 
 const VERSION: u32 = 1;
 
 pub struct OutputMetadataOptions {
-    pub features: Vec<String>,
-    pub no_default_features: bool,
-    pub all_features: bool,
+    pub cli_features: CliFeatures,
     pub no_deps: bool,
     pub version: u32,
     pub filter_platforms: Vec<String>,
@@ -33,7 +33,7 @@ pub fn output_metadata(ws: &Workspace<'_>, opt: &OutputMetadataOptions) -> Cargo
         );
     }
     let (packages, resolve) = if opt.no_deps {
-        let packages = ws.members().cloned().collect();
+        let packages = ws.members().map(|pkg| pkg.serialized()).collect();
         (packages, None)
     } else {
         let (packages, resolve) = build_resolve_graph(ws, opt)?;
@@ -56,7 +56,7 @@ pub fn output_metadata(ws: &Workspace<'_>, opt: &OutputMetadataOptions) -> Cargo
 /// See cargo-metadata.adoc for detailed documentation of the format.
 #[derive(Serialize)]
 pub struct ExportInfo {
-    packages: Vec<Package>,
+    packages: Vec<SerializedPackage>,
     workspace_members: Vec<PackageId>,
     resolve: Option<MetadataResolve>,
     target_directory: PathBuf,
@@ -105,7 +105,7 @@ impl From<&Dependency> for DepKindInfo {
 fn build_resolve_graph(
     ws: &Workspace<'_>,
     metadata_opts: &OutputMetadataOptions,
-) -> CargoResult<(Vec<Package>, MetadataResolve)> {
+) -> CargoResult<(Vec<SerializedPackage>, MetadataResolve)> {
     // TODO: Without --filter-platform, features are being resolved for `host` only.
     // How should this work?
     let requested_kinds =
@@ -113,35 +113,34 @@ fn build_resolve_graph(
     let target_data = RustcTargetData::new(ws, &requested_kinds)?;
     // Resolve entire workspace.
     let specs = Packages::All.to_package_id_specs(ws)?;
-    let resolve_opts = ResolveOpts::new(
-        /*dev_deps*/ true,
-        &metadata_opts.features,
-        metadata_opts.all_features,
-        !metadata_opts.no_default_features,
-    );
+    let force_all = if metadata_opts.filter_platforms.is_empty() {
+        crate::core::resolver::features::ForceAllTargets::Yes
+    } else {
+        crate::core::resolver::features::ForceAllTargets::No
+    };
+
+    // Note that even with --filter-platform we end up downloading host dependencies as well,
+    // as that is the behavior of download_accessible.
     let ws_resolve = ops::resolve_ws_with_opts(
         ws,
         &target_data,
         &requested_kinds,
-        &resolve_opts,
+        &metadata_opts.cli_features,
         &specs,
         HasDevUnits::Yes,
-        crate::core::resolver::features::ForceAllTargets::No,
+        force_all,
     )?;
-    // Download all Packages. This is needed to serialize the information
-    // for every package. In theory this could honor target filtering,
-    // but that would be somewhat complex.
-    let mut package_map: HashMap<PackageId, Package> = ws_resolve
+
+    let package_map: BTreeMap<PackageId, Package> = ws_resolve
         .pkg_set
-        .get_many(ws_resolve.pkg_set.package_ids())?
-        .into_iter()
+        .packages()
         // This is a little lazy, but serde doesn't handle Rc fields very well.
         .map(|pkg| (pkg.package_id(), Package::clone(pkg)))
         .collect();
 
     // Start from the workspace roots, and recurse through filling out the
     // map, filtering targets as necessary.
-    let mut node_map = HashMap::new();
+    let mut node_map = BTreeMap::new();
     for member_pkg in ws.members() {
         build_resolve_graph_r(
             &mut node_map,
@@ -154,27 +153,44 @@ fn build_resolve_graph(
     }
     // Get a Vec of Packages.
     let actual_packages = package_map
-        .drain()
+        .into_iter()
         .filter_map(|(pkg_id, pkg)| node_map.get(&pkg_id).map(|_| pkg))
+        .map(|pkg| pkg.serialized())
         .collect();
+
     let mr = MetadataResolve {
-        nodes: node_map.drain().map(|(_pkg_id, node)| node).collect(),
+        nodes: node_map.into_iter().map(|(_pkg_id, node)| node).collect(),
         root: ws.current_opt().map(|pkg| pkg.package_id()),
     };
     Ok((actual_packages, mr))
 }
 
 fn build_resolve_graph_r(
-    node_map: &mut HashMap<PackageId, MetadataResolveNode>,
+    node_map: &mut BTreeMap<PackageId, MetadataResolveNode>,
     pkg_id: PackageId,
     resolve: &Resolve,
-    package_map: &HashMap<PackageId, Package>,
-    target_data: &RustcTargetData,
+    package_map: &BTreeMap<PackageId, Package>,
+    target_data: &RustcTargetData<'_>,
     requested_kinds: &[CompileKind],
 ) {
     if node_map.contains_key(&pkg_id) {
         return;
     }
+    // This normalizes the IDs so that they are consistent between the
+    // `packages` array and the `resolve` map. This is a bit of a hack to
+    // compensate for the fact that
+    // SourceKind::Git(GitReference::Branch("master")) is the same as
+    // SourceKind::Git(GitReference::DefaultBranch). We want IDs in the JSON
+    // to be opaque, and compare with basic string equality, so this will
+    // always prefer the style of ID in the Package instead of the resolver.
+    // Cargo generally only exposes PackageIds from the Package struct, and
+    // AFAIK this is the only place where the resolver variant is exposed.
+    //
+    // This diverges because the SourceIds created for Packages are built
+    // based on the Dependency declaration, but the SourceIds in the resolver
+    // are deserialized from Cargo.lock. Cargo.lock may have been generated by
+    // an older (or newer!) version of Cargo which uses a different style.
+    let normalize_id = |id| -> PackageId { *package_map.get_key_value(&id).unwrap().0 };
     let features = resolve.features(pkg_id).to_vec();
 
     let deps: Vec<Dep> = resolve
@@ -190,22 +206,23 @@ fn build_resolve_graph_r(
             }
         })
         .filter_map(|(dep_id, deps)| {
-            let dep_kinds: Vec<_> = deps.iter().map(DepKindInfo::from).collect();
+            let mut dep_kinds: Vec<_> = deps.iter().map(DepKindInfo::from).collect();
+            dep_kinds.sort();
             package_map
                 .get(&dep_id)
                 .and_then(|pkg| pkg.targets().iter().find(|t| t.is_lib()))
                 .and_then(|lib_target| resolve.extern_crate_name(pkg_id, dep_id, lib_target).ok())
                 .map(|name| Dep {
                     name,
-                    pkg: dep_id,
+                    pkg: normalize_id(dep_id),
                     dep_kinds,
                 })
         })
         .collect();
-    let dumb_deps: Vec<PackageId> = deps.iter().map(|dep| dep.pkg).collect();
+    let dumb_deps: Vec<PackageId> = deps.iter().map(|dep| normalize_id(dep.pkg)).collect();
     let to_visit = dumb_deps.clone();
     let node = MetadataResolveNode {
-        id: pkg_id,
+        id: normalize_id(pkg_id),
         dependencies: dumb_deps,
         deps,
         features,
