@@ -6,6 +6,7 @@ use crate::util::toml as cargo_toml;
 use crate::util::Filesystem;
 
 use anyhow::Context as _;
+use toml_edit::easy as toml;
 
 pub fn load_pkg_lockfile(ws: &Workspace<'_>) -> CargoResult<Option<Resolve>> {
     if !ws.root().join("Cargo.lock").exists() {
@@ -46,10 +47,6 @@ pub fn write_pkg_lockfile(ws: &Workspace<'_>, resolve: &mut Resolve) -> CargoRes
     }
 
     if !ws.config().lock_update_allowed() {
-        if ws.config().offline() {
-            anyhow::bail!("can't update in the offline mode");
-        }
-
         let flag = if ws.config().network_allowed() {
             "--locked"
         } else {
@@ -58,8 +55,9 @@ pub fn write_pkg_lockfile(ws: &Workspace<'_>, resolve: &mut Resolve) -> CargoRes
         anyhow::bail!(
             "the lock file {} needs to be updated but {} was passed to prevent this\n\
              If you want to try to generate the lock file without accessing the network, \
-             use the --offline flag.",
+             remove the {} flag and use --offline instead.",
             ws.root().to_path_buf().join("Cargo.lock").display(),
+            flag,
             flag
         );
     }
@@ -103,7 +101,7 @@ fn resolve_to_string_orig(
 }
 
 fn serialize_resolve(resolve: &Resolve, orig: Option<&str>) -> String {
-    let toml = toml::Value::try_from(resolve).unwrap();
+    let toml = toml_edit::ser::to_item(resolve).unwrap();
 
     let mut out = String::new();
 
@@ -142,7 +140,7 @@ fn serialize_resolve(resolve: &Resolve, orig: Option<&str>) -> String {
 
     let deps = toml["package"].as_array().unwrap();
     for dep in deps {
-        let dep = dep.as_table().unwrap();
+        let dep = dep.as_inline_table().unwrap();
 
         out.push_str("[[package]]\n");
         emit_package(dep, &mut out);
@@ -152,14 +150,23 @@ fn serialize_resolve(resolve: &Resolve, orig: Option<&str>) -> String {
         let list = patch["unused"].as_array().unwrap();
         for entry in list {
             out.push_str("[[patch.unused]]\n");
-            emit_package(entry.as_table().unwrap(), &mut out);
+            emit_package(entry.as_inline_table().unwrap(), &mut out);
             out.push('\n');
         }
     }
 
     if let Some(meta) = toml.get("metadata") {
-        out.push_str("[metadata]\n");
-        out.push_str(&meta.to_string());
+        // 1. We need to ensure we print the entire tree, not just the direct members of `metadata`
+        //    (which `toml_edit::Table::to_string` only shows)
+        // 2. We need to ensure all children tables have `metadata.` prefix
+        let meta_table = meta
+            .clone()
+            .into_table()
+            .expect("validation ensures this is a table");
+        let mut meta_doc = toml_edit::Document::new();
+        meta_doc["metadata"] = toml_edit::Item::Table(meta_table);
+
+        out.push_str(&meta_doc.to_string());
     }
 
     // Historical versions of Cargo in the old format accidentally left trailing
@@ -193,7 +200,7 @@ fn are_equal_lockfiles(orig: &str, current: &str, ws: &Workspace<'_>) -> bool {
     orig.lines().eq(current.lines())
 }
 
-fn emit_package(dep: &toml::value::Table, out: &mut String) {
+fn emit_package(dep: &toml_edit::InlineTable, out: &mut String) {
     out.push_str(&format!("name = {}\n", &dep["name"]));
     out.push_str(&format!("version = {}\n", &dep["version"]));
 
