@@ -293,8 +293,6 @@ impl<'cfg> RegistryIndex<'cfg> {
     {
         let source_id = self.source_id;
         let config = self.config;
-        let namespaced_features = self.config.cli_unstable().namespaced_features;
-        let weak_dep_features = self.config.cli_unstable().weak_dep_features;
 
         // First up actually parse what summaries we have available. If Cargo
         // has run previously this will parse a Cargo-specific cache file rather
@@ -309,11 +307,6 @@ impl<'cfg> RegistryIndex<'cfg> {
         // minimize the amount of work being done here and parse as little as
         // necessary.
         let raw_data = &summaries.raw_data;
-        let max_version = if namespaced_features || weak_dep_features {
-            INDEX_V_MAX
-        } else {
-            1
-        };
         Ok(summaries
             .versions
             .iter_mut()
@@ -328,7 +321,7 @@ impl<'cfg> RegistryIndex<'cfg> {
                 },
             )
             .filter(move |is| {
-                if is.v > max_version {
+                if is.v > INDEX_V_MAX {
                     debug!(
                         "unsupported schema version {} ({} {})",
                         is.v,
@@ -339,11 +332,6 @@ impl<'cfg> RegistryIndex<'cfg> {
                 } else {
                     true
                 }
-            })
-            .filter(move |is| {
-                is.summary
-                    .unstable_gate(namespaced_features, weak_dep_features)
-                    .is_ok()
             }))
     }
 
@@ -460,19 +448,40 @@ impl<'cfg> RegistryIndex<'cfg> {
         // this source, `<p_req>` is the version installed and `<f_req> is the
         // version requested (argument to `--precise`).
         let name = dep.package_name().as_str();
-        let summaries = summaries.filter(|s| match source_id.precise() {
+        let precise = match source_id.precise() {
             Some(p) if p.starts_with(name) && p[name.len()..].starts_with('=') => {
                 let mut vers = p[name.len() + 1..].splitn(2, "->");
-                if dep
-                    .version_req()
-                    .matches(&vers.next().unwrap().to_semver().unwrap())
-                {
-                    vers.next().unwrap() == s.version().to_string()
+                let current_vers = vers.next().unwrap().to_semver().unwrap();
+                let requested_vers = vers.next().unwrap().to_semver().unwrap();
+                Some((current_vers, requested_vers))
+            }
+            _ => None,
+        };
+        let summaries = summaries.filter(|s| match &precise {
+            Some((current, requested)) => {
+                if dep.version_req().matches(current) {
+                    // Unfortunately crates.io allows versions to differ only
+                    // by build metadata. This shouldn't be allowed, but since
+                    // it is, this will honor it if requested. However, if not
+                    // specified, then ignore it.
+                    let s_vers = s.version();
+                    match (s_vers.build.is_empty(), requested.build.is_empty()) {
+                        (true, true) => s_vers == requested,
+                        (true, false) => false,
+                        (false, true) => {
+                            // Strip out the metadata.
+                            s_vers.major == requested.major
+                                && s_vers.minor == requested.minor
+                                && s_vers.patch == requested.patch
+                                && s_vers.pre == requested.pre
+                        }
+                        (false, false) => s_vers == requested,
+                    }
                 } else {
                     true
                 }
             }
-            _ => true,
+            None => true,
         });
 
         let mut count = 0;
@@ -602,15 +611,13 @@ impl Summaries {
         // present and considered fresh this is where the debug assertions
         // actually happens to verify that our cache is indeed fresh and
         // computes exactly the same value as before.
-        if cfg!(debug_assertions) && cache_contents.is_some() {
-            if cache_bytes != cache_contents {
-                panic!(
-                    "original cache contents:\n{:?}\n\
-                     does not equal new cache contents:\n{:?}\n",
-                    cache_contents.as_ref().map(|s| String::from_utf8_lossy(s)),
-                    cache_bytes.as_ref().map(|s| String::from_utf8_lossy(s)),
-                );
-            }
+        if cfg!(debug_assertions) && cache_contents.is_some() && cache_bytes != cache_contents {
+            panic!(
+                "original cache contents:\n{:?}\n\
+                 does not equal new cache contents:\n{:?}\n",
+                cache_contents.as_ref().map(|s| String::from_utf8_lossy(s)),
+                cache_bytes.as_ref().map(|s| String::from_utf8_lossy(s)),
+            );
         }
 
         // Once we have our `cache_bytes` which represents the `Summaries` we're
@@ -687,7 +694,7 @@ impl Summaries {
 // * `2`: Added the "index format version" field so that if the index format
 //   changes, different versions of cargo won't get confused reading each
 //   other's caches.
-// * `3`: Bumped the version to work around a issue where multiple versions of
+// * `3`: Bumped the version to work around an issue where multiple versions of
 //   a package were published that differ only by semver metadata. For
 //   example, openssl-src 110.0.0 and 110.0.0+1.1.0f. Previously, the cache
 //   would be incorrectly populated with two entries, both 110.0.0. After
