@@ -1,4 +1,4 @@
-use crate::core::compiler::{BuildConfig, MessageFormat};
+use crate::core::compiler::{BuildConfig, MessageFormat, TimingOutput};
 use crate::core::resolver::CliFeatures;
 use crate::core::{Edition, Workspace};
 use crate::ops::{CompileFilter, CompileOptions, NewOptions, Packages, VersionControl};
@@ -15,7 +15,6 @@ use crate::util::{
 use crate::CargoResult;
 use anyhow::bail;
 use cargo_util::paths;
-use clap::{self, SubCommand};
 use std::ffi::{OsStr, OsString};
 use std::path::PathBuf;
 
@@ -23,10 +22,10 @@ pub use crate::core::compiler::CompileMode;
 pub use crate::{CliError, CliResult, Config};
 pub use clap::{AppSettings, Arg, ArgMatches};
 
-pub type App = clap::App<'static, 'static>;
+pub type App = clap::Command<'static>;
 
 pub trait AppExt: Sized {
-    fn _arg(self, arg: Arg<'static, 'static>) -> Self;
+    fn _arg(self, arg: Arg<'static>) -> Self;
 
     /// Do not use this method, it is only for backwards compatibility.
     /// Use `arg_package_spec_no_all` instead.
@@ -55,13 +54,13 @@ pub trait AppExt: Sized {
     }
 
     fn arg_package_spec_simple(self, package: &'static str) -> Self {
-        self._arg(optional_multi_opt("package", "SPEC", package).short("p"))
+        self._arg(optional_multi_opt("package", "SPEC", package).short('p'))
     }
 
     fn arg_package(self, package: &'static str) -> Self {
         self._arg(
             optional_opt("package", package)
-                .short("p")
+                .short('p')
                 .value_name("SPEC"),
         )
     }
@@ -69,7 +68,7 @@ pub trait AppExt: Sized {
     fn arg_jobs(self) -> Self {
         self._arg(
             opt("jobs", "Number of parallel jobs, defaults to # of CPUs")
-                .short("j")
+                .short('j')
                 .value_name("N"),
         )
     }
@@ -142,7 +141,7 @@ pub trait AppExt: Sized {
     }
 
     fn arg_release(self, release: &'static str) -> Self {
-        self._arg(opt("release", release))
+        self._arg(opt("release", release).short('r'))
     }
 
     fn arg_profile(self, profile: &'static str) -> Self {
@@ -212,11 +211,6 @@ pub trait AppExt: Sized {
 
     fn arg_index(self) -> Self {
         self._arg(opt("index", "Registry index URL to upload the package to").value_name("INDEX"))
-            ._arg(
-                opt("host", "DEPRECATED, renamed to '--index'")
-                    .value_name("HOST")
-                    .hidden(true),
-            )
     }
 
     fn arg_dry_run(self, dry_run: &'static str) -> Self {
@@ -233,22 +227,37 @@ pub trait AppExt: Sized {
     fn arg_future_incompat_report(self) -> Self {
         self._arg(opt(
             "future-incompat-report",
-            "Outputs a future incompatibility report at the end of the build (unstable)",
+            "Outputs a future incompatibility report at the end of the build",
         ))
     }
+
+    fn arg_quiet(self) -> Self {
+        self._arg(opt("quiet", "Do not print cargo log messages").short('q'))
+    }
+
+    fn arg_timings(self) -> Self {
+        self._arg(
+            optional_opt(
+                "timings",
+                "Timing output formats (unstable) (comma separated): html, json",
+            )
+            .value_name("FMTS")
+            .require_equals(true),
+        )
+    }
 }
 
 impl AppExt for App {
-    fn _arg(self, arg: Arg<'static, 'static>) -> Self {
+    fn _arg(self, arg: Arg<'static>) -> Self {
         self.arg(arg)
     }
 }
 
-pub fn opt(name: &'static str, help: &'static str) -> Arg<'static, 'static> {
-    Arg::with_name(name).long(name).help(help)
+pub fn opt(name: &'static str, help: &'static str) -> Arg<'static> {
+    Arg::new(name).long(name).help(help)
 }
 
-pub fn optional_opt(name: &'static str, help: &'static str) -> Arg<'static, 'static> {
+pub fn optional_opt(name: &'static str, help: &'static str) -> Arg<'static> {
     opt(name, help).min_values(0)
 }
 
@@ -256,35 +265,25 @@ pub fn optional_multi_opt(
     name: &'static str,
     value_name: &'static str,
     help: &'static str,
-) -> Arg<'static, 'static> {
+) -> Arg<'static> {
     opt(name, help)
         .value_name(value_name)
-        .multiple(true)
+        .multiple_occurrences(true)
+        .multiple_values(true)
         .min_values(0)
         .number_of_values(1)
 }
 
-pub fn multi_opt(
-    name: &'static str,
-    value_name: &'static str,
-    help: &'static str,
-) -> Arg<'static, 'static> {
-    // Note that all `.multiple(true)` arguments in Cargo should specify
-    // `.number_of_values(1)` as well, so that `--foo val1 val2` is
-    // *not* parsed as `foo` with values ["val1", "val2"].
-    // `number_of_values` should become the default in clap 3.
+pub fn multi_opt(name: &'static str, value_name: &'static str, help: &'static str) -> Arg<'static> {
     opt(name, help)
         .value_name(value_name)
-        .multiple(true)
-        .number_of_values(1)
+        .multiple_occurrences(true)
 }
 
 pub fn subcommand(name: &'static str) -> App {
-    SubCommand::with_name(name).settings(&[
-        AppSettings::UnifiedHelpMessage,
-        AppSettings::DeriveDisplayOrder,
-        AppSettings::DontCollapseArgsInUsage,
-    ])
+    App::new(name)
+        .dont_collapse_args_in_usage(true)
+        .setting(AppSettings::DeriveDisplayOrder)
 }
 
 /// Determines whether or not to gate `--profile` as unstable when resolving it.
@@ -304,7 +303,10 @@ pub trait ArgMatchesExt {
         let arg = match self._value_of(name) {
             None => None,
             Some(arg) => Some(arg.parse::<u32>().map_err(|_| {
-                clap::Error::value_validation_auto(format!("could not parse `{}` as a number", arg))
+                clap::Error::raw(
+                    clap::ErrorKind::ValueValidation,
+                    format!("Invalid value: could not parse `{}` as a number", arg),
+                )
             })?),
         };
         Ok(arg)
@@ -316,7 +318,11 @@ pub trait ArgMatchesExt {
     }
 
     fn root_manifest(&self, config: &Config) -> CargoResult<PathBuf> {
-        if let Some(path) = self.value_of_path("manifest-path", config) {
+        if let Some(path) = self
+            ._is_valid_arg("manifest-path")
+            .then(|| self.value_of_path("manifest-path", config))
+            .flatten()
+        {
             // In general, we try to avoid normalizing paths in Cargo,
             // but in this particular case we need it to fix #3586.
             let path = paths::normalize_path(&path);
@@ -391,8 +397,8 @@ pub trait ArgMatchesExt {
         };
 
         let name = match (
-            self._is_present("release"),
-            self._is_present("debug"),
+            self.is_valid_and_present("release"),
+            self.is_valid_and_present("debug"),
             specified_profile,
         ) {
             (false, false, None) => default,
@@ -420,9 +426,13 @@ pub trait ArgMatchesExt {
     fn packages_from_flags(&self) -> CargoResult<Packages> {
         Packages::from_flags(
             // TODO Integrate into 'workspace'
-            self._is_present("workspace") || self._is_present("all"),
-            self._values_of("exclude"),
-            self._values_of("package"),
+            self.is_valid_and_present("workspace") || self.is_valid_and_present("all"),
+            self._is_valid_arg("exclude")
+                .then(|| self._values_of("exclude"))
+                .unwrap_or_default(),
+            self._is_valid_arg("package")
+                .then(|| self._values_of("package"))
+                .unwrap_or_default(),
         )
     }
 
@@ -499,9 +509,37 @@ pub trait ArgMatchesExt {
         let mut build_config = BuildConfig::new(config, self.jobs()?, &self.targets(), mode)?;
         build_config.message_format = message_format.unwrap_or(MessageFormat::Human);
         build_config.requested_profile = self.get_profile_name(config, "dev", profile_checking)?;
-        build_config.build_plan = self._is_present("build-plan");
-        build_config.unit_graph = self._is_present("unit-graph");
-        build_config.future_incompat_report = self._is_present("future-incompat-report");
+        build_config.build_plan = self.is_valid_and_present("build-plan");
+        build_config.unit_graph = self.is_valid_and_present("unit-graph");
+        build_config.future_incompat_report = self.is_valid_and_present("future-incompat-report");
+
+        if self.is_valid_and_present("timings") {
+            for timing_output in self._values_of("timings") {
+                for timing_output in timing_output.split(',') {
+                    let timing_output = timing_output.to_ascii_lowercase();
+                    let timing_output = match timing_output.as_str() {
+                        "html" => {
+                            config
+                                .cli_unstable()
+                                .fail_if_stable_opt("--timings=html", 7405)?;
+                            TimingOutput::Html
+                        }
+                        "json" => {
+                            config
+                                .cli_unstable()
+                                .fail_if_stable_opt("--timings=json", 7405)?;
+                            TimingOutput::Json
+                        }
+                        s => bail!("invalid timings output specifier: `{}`", s),
+                    };
+                    build_config.timing_outputs.push(timing_output);
+                }
+            }
+            if build_config.timing_outputs.is_empty() {
+                build_config.timing_outputs.push(TimingOutput::Html);
+            }
+        }
+
         if build_config.build_plan {
             config
                 .cli_unstable()
@@ -512,44 +550,38 @@ pub trait ArgMatchesExt {
                 .cli_unstable()
                 .fail_if_stable_opt("--unit-graph", 8002)?;
         }
-        if build_config.future_incompat_report {
-            config
-                .cli_unstable()
-                .fail_if_stable_opt("--future-incompat-report", 9241)?;
-
-            if !config.cli_unstable().future_incompat_report {
-                anyhow::bail!(
-                    "Usage of `--future-incompat-report` requires `-Z future-incompat-report`"
-                )
-            }
-        }
 
         let opts = CompileOptions {
             build_config,
             cli_features: self.cli_features()?,
             spec,
             filter: CompileFilter::from_raw_arguments(
-                self._is_present("lib"),
+                self.is_valid_and_present("lib"),
                 self._values_of("bin"),
-                self._is_present("bins"),
-                self._values_of("test"),
-                self._is_present("tests"),
+                self.is_valid_and_present("bins"),
+                self._is_valid_arg("test")
+                    .then(|| self._values_of("test"))
+                    .unwrap_or_default(),
+                self.is_valid_and_present("tests"),
                 self._values_of("example"),
-                self._is_present("examples"),
-                self._values_of("bench"),
-                self._is_present("benches"),
-                self._is_present("all-targets"),
+                self.is_valid_and_present("examples"),
+                self._is_valid_arg("bench")
+                    .then(|| self._values_of("bench"))
+                    .unwrap_or_default(),
+                self.is_valid_and_present("benches"),
+                self.is_valid_and_present("all-targets"),
             ),
             target_rustdoc_args: None,
             target_rustc_args: None,
+            target_rustc_crate_types: None,
             local_rustdoc_args: None,
             rustdoc_document_private_items: false,
-            honor_rust_version: !self._is_present("ignore-rust-version"),
+            honor_rust_version: !self.is_valid_and_present("ignore-rust-version"),
         };
 
         if let Some(ws) = workspace {
             self.check_optional_opts(ws, &opts)?;
-        } else if self.is_present_with_zero_values("package") {
+        } else if self._is_valid_arg("package") && self.is_present_with_zero_values("package") {
             // As for cargo 0.50.0, this won't occur but if someone sneaks in
             // we can still provide this informative message for them.
             anyhow::bail!(
@@ -626,27 +658,8 @@ pub trait ArgMatchesExt {
         }
     }
 
-    fn index(&self, config: &Config) -> CargoResult<Option<String>> {
-        // TODO: deprecated. Remove once it has been decided `--host` can be removed
-        // We may instead want to repurpose the host flag, as mentioned in issue
-        // rust-lang/cargo#4208.
-        let msg = "The flag '--host' is no longer valid.
-
-Previous versions of Cargo accepted this flag, but it is being
-deprecated. The flag is being renamed to 'index', as the flag
-wants the location of the index. Please use '--index' instead.
-
-This will soon become a hard error, so it's either recommended
-to update to a fixed version or contact the upstream maintainer
-about this warning.";
-
-        let index = match self._value_of("host") {
-            Some(host) => {
-                config.shell().warn(&msg)?;
-                Some(host.to_string())
-            }
-            None => self._value_of("index").map(|s| s.to_string()),
-        };
+    fn index(&self) -> CargoResult<Option<String>> {
+        let index = self._value_of("index").map(|s| s.to_string());
         Ok(index)
     }
 
@@ -655,7 +668,7 @@ about this warning.";
         workspace: &Workspace<'_>,
         compile_opts: &CompileOptions,
     ) -> CargoResult<()> {
-        if self.is_present_with_zero_values("package") {
+        if self._is_valid_arg("package") && self.is_present_with_zero_values("package") {
             print_available_packages(workspace)?
         }
 
@@ -667,11 +680,11 @@ about this warning.";
             print_available_binaries(workspace, compile_opts)?;
         }
 
-        if self.is_present_with_zero_values("bench") {
+        if self._is_valid_arg("bench") && self.is_present_with_zero_values("bench") {
             print_available_benches(workspace, compile_opts)?;
         }
 
-        if self.is_present_with_zero_values("test") {
+        if self._is_valid_arg("test") && self.is_present_with_zero_values("test") {
             print_available_tests(workspace, compile_opts)?;
         }
 
@@ -682,6 +695,10 @@ about this warning.";
         self._is_present(name) && self._value_of(name).is_none()
     }
 
+    fn is_valid_and_present(&self, name: &str) -> bool {
+        self._is_valid_arg(name) && self._is_present(name)
+    }
+
     fn _value_of(&self, name: &str) -> Option<&str>;
 
     fn _values_of(&self, name: &str) -> Vec<String>;
@@ -691,9 +708,11 @@ about this warning.";
     fn _values_of_os(&self, name: &str) -> Vec<OsString>;
 
     fn _is_present(&self, name: &str) -> bool;
+
+    fn _is_valid_arg(&self, name: &str) -> bool;
 }
 
-impl<'a> ArgMatchesExt for ArgMatches<'a> {
+impl<'a> ArgMatchesExt for ArgMatches {
     fn _value_of(&self, name: &str) -> Option<&str> {
         self.value_of(name)
     }
@@ -719,13 +738,17 @@ impl<'a> ArgMatchesExt for ArgMatches<'a> {
     fn _is_present(&self, name: &str) -> bool {
         self.is_present(name)
     }
+
+    fn _is_valid_arg(&self, name: &str) -> bool {
+        self.is_valid_arg(name)
+    }
 }
 
-pub fn values(args: &ArgMatches<'_>, name: &str) -> Vec<String> {
+pub fn values(args: &ArgMatches, name: &str) -> Vec<String> {
     args._values_of(name)
 }
 
-pub fn values_os(args: &ArgMatches<'_>, name: &str) -> Vec<OsString> {
+pub fn values_os(args: &ArgMatches, name: &str) -> Vec<OsString> {
     args._values_of_os(name)
 }
 
